@@ -235,6 +235,7 @@ int set_default_options(mod_gm_opt_t *opt) {
     opt->debug_level        = GM_LOG_INFO;
     opt->perfdata           = GM_DISABLED;
     opt->perfdata_mode      = GM_PERFDATA_OVERWRITE;
+    opt->use_uniq_jobs      = GM_ENABLED;
     opt->do_hostchecks      = GM_ENABLED;
     opt->hosts              = GM_DISABLED;
     opt->services           = GM_DISABLED;
@@ -493,6 +494,12 @@ int parse_args_line(mod_gm_opt_t *opt, char * arg, int recursion_level) {
         return(GM_OK);
     }
 
+    /* use_uniq_jobs */
+    else if ( !strcmp( key, "use_uniq_jobs" ) ) {
+        opt->use_uniq_jobs = parse_yes_or_no(value, GM_ENABLED);
+        return(GM_OK);
+    }
+
     else if ( value == NULL ) {
         gm_log( GM_LOG_ERROR, "unknown switch '%s'\n", key );
         return(GM_OK);
@@ -986,7 +993,11 @@ void dumpconfig(mod_gm_opt_t *opt, int mode) {
             gm_log( GM_LOG_DEBUG, "encryption key:      not set\n" );
         }
     }
+    if(mode == GM_NEB_MODE) {
+        gm_log( GM_LOG_DEBUG, "accept clear result: %s\n", opt->accept_clear_results == GM_ENABLED ? "yes" : "no");
+    }
     gm_log( GM_LOG_DEBUG, "transport mode:      %s\n", opt->encryption == GM_ENABLED ? "aes-256+base64" : "base64 only");
+    gm_log( GM_LOG_DEBUG, "use uniq jobs:       %s\n", opt->use_uniq_jobs == GM_ENABLED ? "overwrite" : "append");
 
     gm_log( GM_LOG_DEBUG, "--------------------------------\n" );
     return;
@@ -1141,6 +1152,7 @@ int set_default_job(gm_job_t *job, mod_gm_opt_t *opt) {
     job->timeout             = opt->job_timeout;
     job->start_time.tv_sec   = 0L;
     job->start_time.tv_usec  = 0L;
+    job->has_been_sent       = FALSE;
 
     return(GM_OK);
 }
@@ -1637,6 +1649,12 @@ int check_param_server(gm_server_t * new_server, gm_server_t * server_list[GM_LI
 void send_result_back(gm_job_t * exec_job) {
     gm_log( GM_LOG_TRACE, "send_result_back()\n" );
 
+    /* avoid duplicate returned results */
+    if(exec_job->has_been_sent == TRUE) {
+        return;
+    }
+    exec_job->has_been_sent = TRUE;
+
     if(exec_job->result_queue == NULL) {
         return;
     }
