@@ -54,7 +54,7 @@
 #define MOD_GM_COMMON_H
 
 /* constants */
-#define GM_VERSION                 "1.3.4"
+#define GM_VERSION                 "1.3.8"
 #define GM_ENABLED                      1
 #define GM_DISABLED                     0
 #define GM_BUFFERSIZE               98304
@@ -193,6 +193,7 @@ typedef struct mod_gm_opt_struct {
     int            logmode;                                 /**< logmode: auto, syslog, file or nagios */
     char         * logfile;                                 /**< path for the logfile */
     FILE         * logfile_fp;                              /**< filedescriptor for the logfile */
+    int            use_uniq_jobs;                           /**< flag whether normal jobs will be sent with/without uniq set */
 /* neb module */
     char         * result_queue;                            /**< name of the result queue used by the neb module */
     int            result_workers;                          /**< number of result worker threads started */
@@ -269,6 +270,7 @@ typedef struct gm_job_struct {
     struct timeval core_time;           /**< time when the core started the job */
     struct timeval start_time;          /**< time when the job really started */
     struct timeval finish_time;         /**< time when the job was finished */
+    int            has_been_sent;       /**< flag if job has been sent back */
 } gm_job_t;
 
 
